==> xiome-0.0.8.tgz: package/s/features/administrative/models/administrative-model.ts <==

import {Op, ops} from "../../../framework/ops.js"
import {Service} from "../../../types/service.js"
import {AccessPayload} from "../../auth/types/auth-tokens.js"
import {happystate} from "../../../toolbox/happystate/happystate.js"
import {makeRoleAssignmentService} from "../api/services/role-assignment-service.js"
import {appPermissions} from "../../../assembly/backend/permissions/standard-permissions.js"
import {makeAllowanceChecker} from "../../../assembly/backend/permissions/tools/make-allowance-checker.js"
import {PermissionsDisplay} from "../../auth/aspects/users/routines/permissions/types/permissions-display.js"

export function makeAdministrativeModel({
		roleAssignmentService,
		reauthorize,
	}: {
		roleAssignmentService: Service<typeof makeRoleAssignmentService>
		reauthorize: () => Promise<void>
	}) {

	const {actions, getState, onStateChange} = happystate({
		state: {
			access: <AccessPayload>undefined,
			permissionsOp: <Op<PermissionsDisplay>>ops.none(),
		},
		actions: state => ({
			setAccess(access: AccessPayload) {
				state.access = access
			},
			setPermissionsOp(op: Op<PermissionsDisplay>) {
				state.permissionsOp = op
			},
		}),
	})

	let initializedInDom = false

	function allowanceChecker() {
		return makeAllowanceChecker(getState().access, appPermissions.privileges)
	}

	async function loadPermissions() {
		if (initializedInDom && allowanceChecker()("administrate user roles"))
			await ops.operation({
				promise: roleAssignmentService.fetchPermissions(),
				setOp: actions.setPermissionsOp,
			})
	}

	async function initialize() {
		initializedInDom = true
		await loadPermissions()
	}

	return {
		getState,
		onStateChange,
		initialize,
		get isAllowed() {
			return allowanceChecker()
		},
		accessChange: (access: AccessPayload) => {
			actions.setAccess(access)
			loadPermissions()
		},
		reauthorize,
		searchUsers: roleAssignmentService.searchUsers,
		assignRoleToUser: roleAssignmentService.assignRoleToUser,
		revokeRoleFromUser: roleAssignmentService.revokeRoleFromUser,
	}
}

==> xiome-0.0.12.tgz: package/s/features/administrative/models/administrative-model.ts <==

import {Op, ops} from "../../../framework/ops.js"
import {Service} from "../../../types/service.js"
import {AccessPayload} from "../../auth/types/auth-tokens.js"
import {snapstate} from "../../../toolbox/snapstate/snapstate.js"
import {makeRoleAssignmentService} from "../api/services/role-assignment-service.js"
import {appPermissions} from "../../../assembly/backend/permissions/standard-permissions.js"
import {makeAllowanceChecker} from "../../../assembly/backend/permissions/tools/make-allowance-checker.js"
import {PermissionsDisplay} from "../../auth/aspects/users/routines/permissions/types/permissions-display.js"

export function makeAdministrativeModel({
		roleAssignmentService,
		reauthorize,
	}: {
		roleAssignmentService: Service<typeof makeRoleAssignmentService>
		reauthorize: () => Promise<void>
	}) {

	const state = snapstate({
		accessOp: ops.none() as Op<AccessPayload>,
		permissionsOp: ops.none() as Op<PermissionsDisplay>,
	})

	function getAccess() {
		return ops.value(state.readable.accessOp)
	}

	let initializedInDom = false

	function allowanceChecker() {
		return makeAllowanceChecker(getAccess(), appPermissions.privileges)
	}

	async function loadPermissions() {
		if (initializedInDom && allowanceChecker()("administrate user roles"))
			await ops.operation({
				promise: roleAssignmentService.fetchPermissions(),
				setOp: op => state.writable.permissionsOp = op,
			})
	}

	async function initialize() {
		initializedInDom = true
		await loadPermissions()
	}

	return {
		state: state.readable,
		subscribe: state.subscribe,
		getAccess,
		initialize,
		get isAllowed() {
			return allowanceChecker()
		},
		reauthorize,
		searchUsers: roleAssignmentService.searchUsers,
		assignRoleToUser: roleAssignmentService.assignRoleToUser,
		revokeRoleFromUser: roleAssignmentService.revokeRoleFromUser,
		async updateAccessOp(op: Op<AccessPayload>) {
			state.writable.accessOp = op
			loadPermissions()
		},
	}
}
